import pandas as pd
import os
import datetime
import numpy as np
import sys

# Author J.Connolly USEPA

# Description: Generates drift tables that includes the results for all UDLs used for Step 2. Output includes the
# overlap in each 30 meter drift interval.

# This script has been approved for release by the U.S. Environmental Protection Agency (USEPA). Although
# the script has been subjected to rigorous review, the USEPA reserves the right to update the script as needed
# pursuant to further analysis and review. No warranty, expressed or implied, is made by the USEPA or the U.S.
# Government as to the functionality of the script and related material nor shall the fact of release constitute
# any such warranty. Furthermore, the script is released on condition that neither the USEPA nor the U.S. Government
# shall be held liable for any damages resulting from its authorized or unauthorized use.


# User input variables
chemical_name = ''   # chemical name used for tracking
file_type = 'Range'  # 'Range or CriticalHabitat

# chemical use look up
use_lookup = r"path" + os.sep + chemical_name + "_Uses_lookup_20191104.csv"
# table with on/off use site calls- format specific for table and extension
on_off_excel = r"path\On_Off_calls_June2020.xlsx"
# acres_col = must load path to acres tables - path set on line 105/108
max_drift = '792'
nl48 = True  # boolean True = include the NL48 False = Excludes the NL48

# root path directory
# out tabulated root path - ie Tabulated_[suffix] folder
root_path = r'path\Tabulated_UsageHUCABHabitat'

# Tables directory  where the working/intermediate tables are found; this will start:
# SprayInterval_IntStep_30_MaxDistance_1501
# When combined the root_path + chemical + file_type + folder_path will reach the folder with the working tables
# folder with result table that have no chemical specific adjustments
folder_path = r'SprayInterval_IntStep_30_MaxDistance_1501\noadjust'

# Tables directory  where the working/intermediate tables are found; this will start:
# SprayInterval_IntStep_30_MaxDistance_1501
# When combined the root_path + chemical + file_type + folder_path_census  will reach the folder with the working tables
# that includes the masking from the census of AG and aggregated PCT
# When combined the root_path + chemical + file_type + folder_path_sup  will reach the folder with the working tables
# that includes the masking from the census of AG, aggregated PCT, and limits the overlap to the the extent of the
# supplemental GIS data, e.g Habitat suitability

# census, habitat adjusted, or elevation adjusted or both
folder_path_census = r'SprayInterval_IntStep_30_MaxDistance_1501\census'
folder_path_sup = r'SprayInterval_IntStep_30_MaxDistance_1501\adjHab'  # Set parameter to '' to skip

sup_key = 'HAB'  # term used to identify what the supplemental information is

# TABLE NAMES these are example names for the file name structure- script will loop through all pct groups and all
# treated acres bounding

# Unadjusted tables
BE_interval_table = r"R_UnAdjusted_SprayInterval_noadjust_Full Range_[date].csv"

# # PCT adjusted Tables for census - example tables loops over all tables
# # single example table will loop through all tables; will loop over the all the PCT types (min, max, avg) and the
# # treated acres distributions types for use (Upper, Lower, and Uniform) based on the example table you provide
be_pct_table = "R_Uniform_SprayInterval_Full Range_census_avg_[date].csv"

# #  PCT, On/Off site adjusted Tables for census - example tables loops over all tables
# # If use doesn't want to include on/off use the same tables as the PCT adjusted
BE_sum_PCT_onoff_interval_table = "R_Uniform_SprayInterval_On OffField_census_avg_[date].csv"

# PCT, On/Off site adjusted Tables for census habitat elevation or both- example tables loops over all tables
# If use doesn't want to include on/off use the same tables as the PCT adjusted; supplemental information
BE_sup_table = "R_Lower_SprayInterval_Full Range_adjHab_avg_[date].csv"
BE_onoff_sup_table = "R_Lower_SprayInterval_On OffField_adjHab_avg_[date].csv"
# Variables to use to skip over the supplemental data adjustments
# BE_sup_table =""
# BE_onoff_sup_table = ""

# On/Off Adjustments; True- adjusted the overlap for on/off use site group, False do not adjust the on/off use site
# group
cult_crop = True
orchards_crops = True
pastures = True
residential = True
forest = True
row = True

# master species list
master_list = r"path\MasterListESA_Dec2018_June2020.csv"
# columns from master species list to include in the output tables
col_include_output = ['EntityID', 'Common Name', 'Scientific Name', 'Status', 'pop_abbrev', 'family', 'Lead Agency',
                      'country', 'Group', 'Des_CH', 'CH_GIS', 'WoE Summary Group', 'Critical_Habitat_YesNo', 'Migratory',
                      'Migratory_YesNo', 'CH_Filename', 'Range_Filename', 'L48/NL48']


#  Static variables - set up path - this is done so the path to the chemical does point to a previous run - it is linked
# paths to tables
out_location = root_path
BE_interval = root_path + os.sep + chemical_name + os.sep + file_type + os.sep + folder_path + os.sep + BE_interval_table
BE_sum_PCT_path = root_path + os.sep + chemical_name + os.sep + file_type + os.sep + folder_path_census
BE_sum_PCT_path_sup = root_path + os.sep + chemical_name + os.sep + file_type + os.sep + folder_path_sup

# file type - Critical habitat or R- because R and critical habitat files are split is separate folders even if
# script is started without updating the critical hab and range path the script will **not** run because the files
# nape will not start with the correct file_type identifier
find_file_type = os.path.basename(BE_interval)
if find_file_type.startswith('R'):
    file_type_marker = 'R_'
    acres_col = r"C:\Users\JConno02\Environmental Protection Agency (EPA)\Endangered Species Pilot Assessments - Herbicide BEs 2020\Overlap Inputs\R_Acres_Pixels_20200628.csv"
else:
    file_type_marker = 'CH_'
    acres_col = r"C:\Users\JConno02\Environmental Protection Agency (EPA)\Endangered Species Pilot Assessments - Herbicide BEs 2020\Overlap Inputs\CH_Acres_Pixels_20200628.csv"


if BE_onoff_sup_table != '':
    table_loop = [BE_sum_PCT_onoff_interval_table, BE_onoff_sup_table]
else:
    table_loop = [BE_sum_PCT_onoff_interval_table]


def create_directory(dbf_dir):
    # Create output directories
    if not os.path.exists(dbf_dir):
        os.mkdir(dbf_dir)


def save_parameter(df, chemicalname, filetype, uselookup, masterlist, no_adjust_p, pct_p, pct_onff_p, outbase,
                   index_num,acres):
    # Function to save/track input parameters used
    df.loc[index_num, 'Chemical Name'] = chemicalname
    df.loc[index_num, 'File Type'] = filetype
    df.loc[index_num, 'Use Lookup'] = uselookup
    df.loc[index_num, 'Species List'] = masterlist
    df.loc[index_num, 'Acres'] = acres
    df.loc[index_num, 'In Location No Adjust'] = no_adjust_p
    df.loc[index_num, 'In Location PCT Adjust'] = pct_p
    df.loc[index_num, 'In Location PCT/On Off Adjust'] = pct_onff_p
    df.loc[index_num, 'Out file'] = 'Step 2 Interval'
    df.loc[index_num, 'Out Base Location'] = outbase
    return df


def on_off_field(cols, df, on_off_species, df_not_on_off, acres):
    # Adjust the drift summarized calls for the on/off site cols
    l48_cols = [p for p in cols if p.startswith('CONUS')]
    nl48_cols = [p for p in cols if not p.startswith('CONUS')]

    species_acres = acres.loc[acres['EntityID'].isin(on_off_species)]
    conus_acres = species_acres[["EntityID", "Acres_CONUS"]]
    nl48_acres = species_acres[["EntityID", "TotalAcresNL48"]]

    species_unadjusted = df_not_on_off.loc[df_not_on_off['EntityID'].isin(on_off_species)]
    # back out acres for use
    species_unadjusted.ix[:, cols] = species_unadjusted.ix[:, cols].div(100)
    # conus
    species_unadjusted = pd.merge(species_unadjusted, conus_acres, on='EntityID', how='left')
    species_unadjusted.ix[:, l48_cols] = species_unadjusted.ix[:, l48_cols].multiply(
        species_unadjusted["Acres_CONUS"].where(species_unadjusted["Acres_CONUS"] != 0, np.nan), axis=0)
    col_direct_conus = [v for v in l48_cols if v.endswith('_0')]
    # update the overlap numbers after removing the direct overlap from the range
    for col in col_direct_conus:
        species_unadjusted["CONUS_Adjusted"] = species_unadjusted["Acres_CONUS"].sub(
            species_unadjusted[col].where(species_unadjusted["Acres_CONUS"] != 0, np.nan), fill_value=0)
        drift_cols = [v for v in l48_cols if v.startswith(col.split("_")[0] + "_" + col.split("_")[1])]
        drift_cols = [v for v in drift_cols if not v.endswith('_0')]
        species_unadjusted.ix[:, drift_cols] = species_unadjusted.ix[:, drift_cols].div(
            species_unadjusted["CONUS_Adjusted"].where(species_unadjusted["Acres_CONUS"] != 0, np.nan), axis=0)
        species_unadjusted.ix[:, drift_cols] *= 100

    # nl48
    if nl48:
        species_unadjusted.drop("Acres_CONUS", axis=1, inplace=True)
        species_unadjusted = pd.merge(species_unadjusted, nl48_acres, on='EntityID', how='left')
        species_unadjusted.ix[:, nl48_cols] = species_unadjusted.ix[:, nl48_cols].multiply(
            species_unadjusted["TotalAcresNL48"].where(species_unadjusted["TotalAcresNL48"] != 0, np.nan), axis=0)
        col_direct_nl48 = [v for v in nl48_cols if v.endswith('_0')]
        for col in col_direct_nl48:
            species_unadjusted["NL48_Adjusted"] = species_unadjusted["TotalAcresNL48"].sub(
                species_unadjusted[col].where(species_unadjusted["TotalAcresNL48"] != 0, np.nan), fill_value=0)
            drift_cols = [v for v in nl48_cols if v.startswith(col.split("_")[0] + "_" + col.split("_")[1])]
            drift_cols = [v for v in drift_cols if not v.endswith('_0')]
            species_unadjusted.ix[:, drift_cols] = species_unadjusted.ix[:, drift_cols].div(
                species_unadjusted["NL48_Adjusted"].where(species_unadjusted["TotalAcresNL48"] != 0, np.nan), axis=0)
            species_unadjusted.ix[:, drift_cols] *= 100

    df_all_drift = [v for v in cols if not v.endswith("_0")]
    drift_df = species_unadjusted[["EntityID"] + df_all_drift]
    drift_df.ix[:, 'EntityID'] = drift_df.ix[:, 'EntityID'].map(lambda r: str(r).split('.')[0]).astype(str)
    left_update = df.set_index('EntityID')
    right_update = drift_df.set_index('EntityID')

    res = left_update.reindex(columns=left_update.columns.union(right_update.columns))
    res.update(right_update)

    df = res.reset_index()

    return df


def extract_chemical_overlap(col_prefix, region, df, distance_bins, section):
    # Generated a list of columns that will be found in the overlap tables based on the grounds and aerial values from
    # the chemical use look up table - we can specify ground or aerial based on chemical labels; direct overlap is
    # always included
    global out_df
    col_selection = ['EntityID']
    for col in col_prefix:
        if region == 'CONUS':
            if col.split("_")[0] == 'CONUS':
                for distance in distance_bins:
                    col_selection.append(col + "_" + str(distance))
        else:
            if col.split("_")[0] != 'CONUS':
                for distance in distance_bins:
                    col_selection.append(col + "_" + str(distance))
    try:
        # Extracts the use columns from the overlap table
        df = df.ix[:, col_selection]
        df = df.reindex(columns=col_selection)
    except:
        print ('A column from the chemical use lookup table is not in the overlap table; check the use lookup table '
               'and restart; error in section {0}').format(section)
        print col_selection
        print df.columns.values.tolist()
        print ('Check that a use drop because all species has 0 overlap - this has happened in AK')
        sys.exit()
    # Summarizes the individual overlap in the NL48 regions to a summarized values for all NL48 regions

    # SUM INDIVIDUAL NL48 REGION TO ONE GROUPED VALUE FOR ALL NL48
    if region == 'CONUS':
        col_region = 'CONUS_'
    else:
        col_region = 'NL48_'
    if col_region == 'NL48_':
        binned_use = []
        for x in col_selection:
            if x in col_include_output:
                pass
            else:
                if x.split("_")[1] not in binned_use:
                    binned_use.append(x.split("_")[1])
        out_col = []
        for bin_col in binned_use:
            list_col = [v for v in df.columns.values.tolist() if bin_col in v.split("_")]
            for interval in distance_bins:
                # confirm it is _0 and not just endswith; everything ends with 0 so without the _ all distances will be
                # in the _0 result
                list_col_interval = [z for z in list_col if z.endswith("_" + str(interval))]
                # print interval, list_col_interval
                df = df.apply(pd.to_numeric, errors='coerce')
                df[col_region + list_col_interval[0].split("_")[1] + "_" + list_col_interval[0].split("_")[2]] = df[
                    list_col_interval].sum(axis=1)
            out_col = [v for v in df.columns.values.tolist() if v.startswith(col_region)]
            out_col.insert(0, 'EntityID')
            out_df = df.ix[:, out_col]
            out_df.ix[:, 'EntityID'] = out_df.ix[:, 'EntityID'].map(lambda r: str(r).split('.')[0]).astype(str)
    else:
        out_df = df.copy()

    return out_df, col_selection


def apply_factor(factor_df, pct_df, skip_non_ag_adjustment, use_direct_only_conus_ag, use_direct_only_nl48_ag,
                 use_direct_only_conus_ag_aa, use_direct_only_nl48_ag_aa, use_direct_only_conus_nonag,
                 use_direct_only_nl48_nonag, use_direct_only_conus_nonag_aa, use_direct_only_nl48_nonag_aa,
                 overlap_cols_conus, overlap_cols_nl48):
    # Load factors (calculated from the un-adjusted overlap) into the PCT overlap table
    if not skip_non_ag_adjustment:
        if nl48:
            factors_df = factor_df.ix[:, ['EntityID', 'CONUS_Ag_Ag_Factor', 'CONUS_NonAg_NonAg_Factor', 'NL48_Ag_Ag_Factor',
                                          'NL48_NonAg_NonAg_Factor', 'CONUS_Composite_Factor',
                                          'NL48_Composite_Factor']].fillna(0)
        else:
            factors_df = factor_df.ix[:, ['EntityID', 'CONUS_Ag_Ag_Factor', 'CONUS_NonAg_NonAg_Factor',
                                          'CONUS_Composite_Factor']].fillna(0)


    else:
        if nl48:
            factors_df = factor_df.ix[:, ['EntityID', 'CONUS_Ag_Ag_Factor', 'NL48_Ag_Ag_Factor', 'CONUS_Composite_Factor',
                                          'NL48_Composite_Factor']].fillna(0)
        else:
            factors_df = factor_df.ix[:, ['EntityID', 'CONUS_Ag_Ag_Factor', 'CONUS_NonAg_NonAg_Factor',
                                          'CONUS_Composite_Factor',]].fillna(0)

    pct_df = pd.merge(pct_df, factors_df, on='EntityID', how='left')

    # # Applies the factor adjustments to the pct adjusted overlap
    # # APPLIES COMPOSITE FACTOR

    # To Ag uses layer
    pct_df.ix[:, use_direct_only_conus_ag] = pct_df.ix[:, use_direct_only_conus_ag].div(
        pct_df['CONUS_Composite_Factor'].where(pct_df['CONUS_Composite_Factor'] != 0, np.nan), axis=0)
    if nl48:
        pct_df.ix[:, use_direct_only_nl48_ag] = pct_df.ix[:, use_direct_only_nl48_ag].div(
            pct_df['NL48_Composite_Factor'].where(pct_df['NL48_Composite_Factor'] != 0, np.nan), axis=0)
    # To composites
    pct_df.ix[:, use_direct_only_conus_ag_aa] = pct_df.ix[:, use_direct_only_conus_ag_aa].div(
        pct_df['CONUS_Composite_Factor'].where(pct_df['CONUS_Composite_Factor'] != 0, np.nan), axis=0)
    if nl48:
        pct_df.ix[:, use_direct_only_nl48_ag_aa] = pct_df.ix[:, use_direct_only_nl48_ag_aa].div(
            pct_df['NL48_Composite_Factor'].where(pct_df['NL48_Composite_Factor'] != 0, np.nan), axis=0)
